package au.com.postnewspapers.pgclientcertdemo;

import java.sql.SQLException;
import org.apache.commons.lang.exception.ExceptionUtils;

/**
 * Demonstrate the use of PgJDBC with client certificates.
 *
 * Current PostgreSQL (8.4, 9.0-pre) do not send the list of accepted
 * roots to a client when issuing a CertificateRequest. This means the client
 * can't pick which client certificate to send to the server.
 *
 * In Java it's possible to override the default SSL negotiation infrastructure,
 * forcing presentation of a particular client certificate without checking what
 * the server wants/expects. Doing so, however, removes your access to the automatic
 * extensibility built in to the Sun crypto implementation (JSSE), so you don't get
 * automatic PKCS#11 hardware token support, etc. It's also more code to maintain.
 *
 * If the Pg server told the client which CAs it trusted to sign client certs,
 * this would be unnecessry.
 *
 * This app will do client cert negotiation with an unpatched Pg server using
 * a custom SSLSocketFactory and X509KeyManager. It'll work with the default
 * SSLSocketFactory if the Pg server is patched to send the trusted root list.
 */
public class App 
{
    public static void main( String[] args )
    {
        if (args.length != 5) {
            exitWithUsage();
        }
        // Select socket factory to use
        boolean useCustomFactory = false;
        if (args[0].equals("custom")) {
            System.out.println("Using custom SSL socket factory");
            useCustomFactory = true;
        } else if (!args[0].equals("default")) {
            System.out.println("Mode must be 'custom' or 'default', not " + args[0]);
            exitWithUsage();
        }
        // Tell SSLSocketFactory where to find key data
        System.setProperty("javax.net.ssl.keyStoreType", args[1]);
        System.setProperty("javax.net.ssl.keyStore", args[2]);
        System.setProperty("javax.net.ssl.keyStorePassword", args[3]);
        // Tell user where we're getting certificate trust data from
        String trustStore = System.getProperty("javax.net.ssl.trustStore");
        if (trustStore == null || trustStore.equals("")) {
            // Java default trust store
            trustStore = System.getProperty("java.home") + "/lib/security/cacerts";
        }
        System.out.println("Using trusted root store: " + trustStore);
        // Enable tracing in JSSE SSL engine
        System.setProperty("javax.net.debug", "ssl:handshake:trustmanager:keymananager");
        // and kick off the test
        TestConnection test = new TestConnection();
        test.setURL(args[4]);
        if (useCustomFactory) {
            test.setSSLSocketFactoryClass("au.com.postnewspapers.ssl.CustomSSLSocketFactory");
        }
        try {
            String vers = test.getServerVersion();
            System.out.println("Connection and query successful, server version is " + vers);
        } catch (ClassNotFoundException ex) {
            System.out.println("Unable to load JDBC driver");
            ex.printStackTrace();
        } catch (SQLException ex) {
            // A bad keystore passphrase results in a deeply nested javax.crypto.BadPaddingException
            // Go figure
            int idx = ExceptionUtils.indexOfType(ex, javax.crypto.BadPaddingException.class);
            if (idx != -1) {
                System.out.println("Unable to open key store unable to extract key. Bad password?");
                System.exit(1);
            } else {
                System.out.println("Connection or query failed");
                ex.printStackTrace();
            }
        }
    }

    private static void exitWithUsage() {
        System.out.println(
                "Usage: java -jar PgClientCertDemo.jar (custom|std) \\\n" +
                "                 storetype storepath storepass jdbc_url\n" +
                "\n" +
                "where: (custom|default)    Whether to use a custom cert factory or\n" +
                "                           stick to the standard one.\n" +
                "       storetype           The keystore type, generally 'pkcs12' or 'jks'\n" +
                "       storepath           Location of keystore in file system\n" +
                "       storepass           Password store is secured with.\n" +
                "                           For jks stores, also the key pass.\n" +
                "       jdbc_url            JDBC URL to establish connection to\n" +
                "\n" +
                "   eg: java -jar PgClientCertDemo.jar std jks test.jks passforstore someurl\n" +
                "   eg: java -jar PgClientCertDemo.jar custom pkcs12 test.p12 testpass otherurl\n" +
                "\n" +
                "NOTE: You must specify the username and password on the JDBC URL using the \n" +
                "      user= and password= parameters, eg: \n" +
                "         jdbc:postgresql://hostname/dbname?ssl=true&user=bob&password=test\n" +
                "      If you do not supply an explicit ssl= parameter, an unencrypted connection\n" +
                "      will be made and no client cert negotiation will take place.\n" +
                "      It is *not* permitted to supply an sslfactory= parameter.\n" +
                "\n" +
                "NOTE: If you need to add additional trusted server certificates,\n" +
                "      add them to the Java trust store in JAVA_HOME/lib/security/cacerts" +
                "      using keytool, or specify a trustStore with the \n" +
                "      javax.net.ssl.trustStore, .trustStoreType and .trustStorePassword\n" +
                "      parameters and install your cert into that using keytool.\n" +
                "      See http://java.sun.com/javase/6/docs/technotes/guides/security/jsse/JSSERefGuide.html\n" +
                "      The default password for the 'cacerts' keystore is 'changeme'\n." +
                "\n" +
                "  eg: keytool -storetype jks -keystore trusted.jks -storepass password -importCert trusted.pem\n" +
                "      java -Djavax.net.ssl.trustStore=trusted.jks\\\n" +
                "           -Djavax.net.ssl.trustStorePassword=password\\\n" +
                "           -Djavax.net.ssl.trustStoreType=jks\\\n" +
                "           -jar PgClientCertDemo.jar custom pkcs12 mykey.p12 mypass\\\n" +
                "           'jdbc:postgresql://servername/db?user=fred&password=userpass&ssl=true'\n" +
                "\n"
                );
        System.exit(0);
    }
}
